/**
 * Javascript for time tracking.
 */

// The update interval and seconds to milliseconds divider.
const INTERVAL = 1000;
const MILLISECONDS = 1000;
const UPDATE_INTERVAL = 100; // Update every 100ms.

// Check for the drupal settings.
if (drupalSettings.hasOwnProperty('openlucius_timetracker')) {

  // Check the node ip and port.
  if (drupalSettings.openlucius_timetracker.hasOwnProperty('node_ip') && drupalSettings.openlucius_timetracker.hasOwnProperty('node_port')) {

    var node_ip = drupalSettings.openlucius_timetracker.node_ip;
    var node_port = drupalSettings.openlucius_timetracker.node_port;

    // Connect to the nodeJs server.
    var socket = io.connect('http://' + drupalSettings.openlucius_timetracker.node_ip + ':' + drupalSettings.openlucius_timetracker.node_port);

    // Temporary logging.
    console.log('Connected to: ' + 'http://' + node_ip + ':' + node_port);
  }
}

(function ($, drupalSettings) {

  // The timetracker jquery.
  Drupal.behaviors.timetracker = {
    attach: function (context, settings) {

      /**
       * - When the document loads place the current time if the timer is running.
       * - Place the stop button if the timer is running.
       * - Place another start button if you're on another page.
       * - Place the start button if the timer is not running.
       */

      var uid = drupalSettings.openlucius_timetracker.uid;

      // Refresh the timer which calls 'server refresh timer'.
      socket.emit('client refresh timer', uid);

      //
      var startStopBtn = $('.timetracker-start-stop-button');

      if (startStopBtn.length > 0) {

        // If the button has the class fa-stop, it is currently running.
        // therefore we need to do:
        // socket.emit('client start timer', nid, uid, new Date().getTime(), 'channel title');

        startStopBtn.off().on('mousedown', function () {

          // Trigger start if the button has the fa-play class.
          if (startStopBtn.hasClass('fa-play')) {

            // Toggle the button.
            startStopBtn.addClass('fa-stop').removeClass('fa-play');

            // Send a socket emit to start the timer.
            clientStartTimer();
          } else {

            // Trigger stop if the button has the fa-stop class.
            if (startStopBtn.hasClass('fa-stop')) {

              // Toggle the button.
              startStopBtn.addClass('fa-play').removeClass('fa-stop');

              // Send a socket emit to stop the timer.
              clientStopTimer();
            }
          }
        });
      }
    }
  };

  var stopwatchInterval = 0;

  /**
   * Refresh the timer.
   */
  function serverRefreshTimer(entry) {

    // Initialize variables.
    var cid = 0;
    var title = '';
    var start = 0;
    var seconds = 0;

    var currentNid = drupalSettings.openlucius_timetracker.nid;

    if (typeof(entry && entry.cid) !== 'undefined') {
      cid = entry.cid;
    }
    else {
      cid = currentNid;
    }

    // Typecast both to numbers.
    currentNid = parseInteger(currentNid);
    cid = parseInteger(cid);

    // Check for the right node id. (Can be string & number).
    if (currentNid === cid) {

      // If there's no entry, create it based on the current details.
      if (typeof(entry) === 'undefined') {

        // Get the time.
        var time = $('.time-entry-time').html();
        var current = moment(time, 'HH:mm:ss').format('X');
        var now = createUnixTimestamp();
        seconds = now - current;
      }
      else {
        title = entry.title;
        start = entry.start;
        seconds = createUnixTimestamp() - entry.start;
      }

      // If there is an entry, set the stop button and start the stopwatch
      // where it left off.
      var startStopBtn = $('.timetracker-start-stop-button');
      startStopBtn.removeClass('fa-play').addClass('fa-stop');
      var formatted = secondsToHumanReadable(seconds);

      /**
       * Step 2.
       */

      // Set the time and title.
      $('.time-entry-title').html(title);
      $('.time-entry-time').html(formatted); // Correct.

      // Start ticking time from the elapsed time point.
      timerStartCounting(start, seconds);
    }
  }

  /**
   * Start the timer.
   */
  function clientStartTimer() {

    var nid = drupalSettings.openlucius_timetracker.nid;
    var uid = drupalSettings.openlucius_timetracker.uid;
    var title = drupalSettings.openlucius_timetracker.title;

    // Get the time.
    var time = $('.time-entry-time').html();
    var seconds = convertToSeconds(time); // Correct.

    /**
     * Step 1.
     */

    // Check if there is a timer running, if so, stop that timer.
    timerStartCounting(0, seconds);

    // Emit the start timer.
    var start = createUnixTimestamp();
    socket.emit('client start timer', nid, uid, start, title);
  }

  /**
   * Start counting.
   *
   * @param {int} start
   *   The start timestamp.
   * @param {int} elapsed
   *   The time that has elapsed already.
   */
  function timerStartCounting(start, elapsed) {

    if (typeof start === 'undefined' || !start) {
      start = 0;
    }

    // Clear the interval.
    clearInterval(stopwatchInterval);

    if (typeof elapsed === 'undefined' || !elapsed) {
      elapsed = 0;
    }

    // Create the stopwatch interval.
    stopwatchInterval = setInterval(function () {

      var now = createUnixTimestamp();
      var timeElapsed = 0;

      if (start !== 0) {
        timeElapsed = now - start;
      }
      else {
        timeElapsed = now - start - elapsed;
      }

      // Calculate the time.
      //var timeElapsed = now - start - elapsed;
      var formattedTime = moment.unix(timeElapsed).utc().format("HH:mm:ss");

      // Update the display.
      $('.time-entry-time').html(formattedTime);
    }, UPDATE_INTERVAL);
  }

  /**
   * Stop the timer.
   */
  function clientStopTimer() {

    var nid = drupalSettings.openlucius_timetracker.nid;
    var uid = drupalSettings.openlucius_timetracker.uid;
    var my_time = drupalSettings.openlucius_timetracker.my_time;

    // Emit the stop timer.
    socket.emit('client stop timer', nid, uid);

    // Stop the stopwatch.
    timerStopCounting();

    // Get the time.
    var time = $('.time-entry-time').html();
    var old = moment(my_time, 'HH:mm:ss').format('X');
    var current = moment(time, 'HH:mm:ss').format('X');
    var now = createUnixTimestamp();

    // Create the start timestamp.
    var start = now - (current - old);

    finishTimeEntry(nid, uid, start, 'Automatic time entry.');

    // Temporary reloading.
    //location.reload();
  }

  /**
   * Stop counting.
   */
  function timerStopCounting() {

    // Stop the interval.
    clearInterval(stopwatchInterval);

    // Change the class t
    var startStopBtn = $('.timetracker-start-stop-button');
    startStopBtn.removeClass('fa-stop').addClass('fa-play');

    // Reload for debugging.
    location.reload();
  }

  /**
   * Create the current unix timestamp.
   * @returns {number}
   *   Returns the current unix timestamp as number.
   */
  function createUnixTimestamp() {
    return moment().unix() - 3600;
  }

  /**
   * Convert a timestamp string to seconds.
   *
   * @param {string} str
   *   The string to convert.
   * @returns {number}
   *   Returns the string as number format.
   */
  function convertToSeconds(str) {
    var p = str.split(':'),
      s = 0,
      m = 1;

    while (p.length > 0) {
      s += m * parseInt(p.pop(), 10);
      m *= 60;
    }

    return s;
  }

  /**
   * Convert value to integer.
   *
   * @param value
   *   The value to convert.
   * @returns {number}
   *   Returns the value as number.
   */
  function parseInteger(value) {
    if (value === '') return NaN;
    const number = Number(value);
    return Number.isInteger(number) ? number : NaN;
  }

  /**
   * Convert UTC time with timezone offset.
   *
   * @returns {number}
   *   Returns timezone as number.
   */
  Date.prototype.getUTCTime = function () {
    return this.getTime() - (this.getTimezoneOffset() * 60000);
  };

  /**
   * Get a formatted date.
   *
   * @returns {string}
   *   Returns the formatted date as string.
   */
  function getFormattedDate() {
    return moment().format('hh:mm:ss');
  }

  /**
   * Convert seconds to human readable hours:minutes:seconds.
   *
   * @param seconds
   *   The seconds to convert.
   *
   * @return {string}
   *   Returns a human readable string.
   */
  function secondsToHumanReadable(seconds) {
    return moment.duration(seconds, 'seconds').format();
  }

  /**
   * Format the duration to hours minutes seconds.
   *
   * @returns {string}
   *   Returns the formatted string.
   */
  moment.duration.fn.format = function () {

    var hours = ('0' + this.days()).slice(-2);
    var minutes = ('0' + this.hours()).slice(-2);
    var seconds = ('0' + this.minutes()).slice(-2);

    var string = hours + ':' + minutes + ':' + seconds;

    return string;
  };


  /**
   * ===========================================================================
   * Ajax functions.
   * ===========================================================================
   */

  /**
   * Finish and save a time entry.
   *
   * @param {int} nid
   *   The time entry node id.
   * @param {int} uid
   *   The time entry user id.
   * @param {int} start
   *   The time entry start timestamp.
   * @param {string} message
   *   The time entry message.
   */
  function finishTimeEntry(nid, uid, start, message) {

    // Get the current time to use as stop time.
    var stop = createUnixTimestamp();

    $.ajax({
      type: 'GET',
      contentType: "application/json",
      url: drupalSettings.basepath + 'create-time-entry',
      dataType: 'json',
      data: {
        _format: 'json',
        nid: nid,
        uid: uid,
        start: start,
        stop: stop,
        message: message
      }
    });
  }

  /**
   * ===========================================================================
   * NodeJS communication logic.
   * ===========================================================================
   */

  /**
   * When 'client refresh timer' triggers a result it's given here.
   */
  socket.on('server refresh timer', function (entry) {
    serverRefreshTimer(entry);
  });

  /**
   * Start the timer everywhere.
   */
  socket.on('server start time', function () {
    serverRefreshTimer();
  });

  socket.on('server stop time', function (e) {
    timerStopCounting();
  })

}(jQuery, drupalSettings));
