<?php

namespace Drupal\openlucius_timetracker\Form;

use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\BeforeCommand;
use Drupal\Core\Ajax\HtmlCommand;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Class TimetrackerForm.
 *
 * @package Drupal\openlucius_timetracker\Form
 */
class TimetrackerForm extends FormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'timetracker_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {

    // Get the services.
    $timetrackerService = \Drupal::service('openlucius_timetracker.timetracker_service');

    // Get the current node id.
    $nid = $timetrackerService->getCurrentNodeId();

    // Set the hidden node id and user id.
    $form['timetracker_nid'] = [
      '#type' => 'hidden',
      '#value' => $nid,
    ];
    $form['timetracker_uid'] = [
      '#type' => 'hidden',
      '#value' => \Drupal::currentUser()->id(),
    ];
    $form['timetracker_current_time'] = [
      '#type' => 'hidden',
      '#value' => $timetrackerService->getTotalTime($nid),
    ];
    // The select for the type (negative / positive).
    $form['timetracker_type'] = [
      '#type' => 'select',
      '#title' => $this->t('Type'),
      '#options' => [
        '-' => $this->t('+'),
        '+' => $this->t('-'),
      ],
      '#default_value' => 0,
      '#prefix' => '<div class="form-group text-left col-lg-3">',
      '#suffix' => '</div>',
      '#attributes' => [
        'class' => [
          'form-control',
        ],
      ],
    ];
    // The number field for the amount of hours.
    $form['timetracker_hours'] = [
      '#type' => 'number',
      '#title' => $this->t('Hours'),
      '#maxlength' => 3,
      '#min' => 0,
      '#step' => 1,
      '#prefix' => '<div class="form-group text-left col-lg-3">',
      '#suffix' => '</div>',
      '#attributes' => [
        'class' => [
          'form-control',
        ],
        //'placeholder' => t('1'),
      ],
      //'#default_value' => '1',
    ];
    // The number field for the amount of minute.
    $form['timetracker_minutes'] = [
      '#type' => 'number',
      '#title' => $this->t('Minutes'),
      '#maxlength' => 2,
      '#min' => 0,
      '#max' => 59,
      '#step' => 1,
      '#prefix' => '<div class="form-group text-left col-lg-3">',
      '#suffix' => '</div>',
      '#attributes' => [
        'class' => [
          'form-control',
        ],
        //'placeholder' => t('30'),
      ],
      //'#default_value' => '0',
    ];
    // The textfield for a small message.
    $form['timetracker_message'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Message'),
      '#prefix' => '<div class="form-group text-left no-label col-lg-3">',
      '#suffix' => '</div>',
      '#attributes' => [
        'class' => [
          'form-control',
        ],
        'placeholder' => t('Message ...'),
      ],
      '#required' => TRUE, // One should always provide a message.
      //'#default_value' => 'Testing ...',
    ];
    // The submit button with an AJAX callback to process the data.
    $form['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Add time entry'),
      '#ajax' => [
        'callback' => [$this, 'respondToAjax'],
        'event' => 'click',
        'progress' => [
          'type' => 'throbber',
          'message' => $this->t('Please wait...'),
        ],
      ],
      // Has a label-hidden to add some margin-top.
      '#prefix' => '<div class="form-group text-left /*label-hidden*/ col-lg-12">',
      '#suffix' => '</div>',
      '#attributes' => [
        'class' => [
          'form-control',
          'btn-success',
          'add-time-entry-btn',
        ],
      ],
    ];

    // Disable caching on this form.
    $form_state->setCached(FALSE);
    $form['#cache']['max-age'] = 0;

    return $form;
  }

  /**
   * AJAX response: alert contents of input field, update form and save it.
   *
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   * Form state information.
   *
   * @return \Drupal\Core\Ajax\AjaxResponse
   */
  public static function respondToAjax(array &$form, FormStateInterface $form_state) {

    drupal_set_message();

    // Get the services and create the time entry.
    $timetrackerService = \Drupal::service('openlucius_timetracker.timetracker_service');

    // Create a new AjaxResponse.
    $ajax_response = new AjaxResponse();

    // If errors, show the messages.
    if ($form_state->hasAnyErrors()) {

      // Get messages even if not shown.
      $status_messages = array('#type' => 'status_messages');
      $message = \Drupal::service('renderer')->renderRoot($status_messages);
      $ajax_response->addCommand(new HtmlCommand('#timetracker-feedback', $message));

      // Return the response.
      return $ajax_response;
    }

    $nid = $form_state->getValue('timetracker_nid');
    $uid = $form_state->getValue('timetracker_uid');

    $type = $form_state->getValue('timetracker_type');
    $hours = $form_state->getValue('timetracker_hours');
    $minutes = $form_state->getValue('timetracker_minutes');
    $message = $form_state->getValue('timetracker_message');

    // Calculate the extra time in seconds.
    $extra_time = ($hours * 3600) + ($minutes * 60);

    $start = 0;
    $stop = time();

    // Todo - Check plus and minus!
    if ($type == '-') {
      $start = $stop - $extra_time;
    }
    elseif ($type == '+') {
      $start = $stop + $extra_time;
    }

    // Create the time entry and create / update the day totals.
    $timetrackerService->createTimeEntry($nid, $uid, $start, $stop, $message);
    $timetrackerService->createUserDayTotal($uid, $extra_time);

    // - Set a success message in stead of
    $current_total_time = $form_state->getValue('timetracker_current_time');
    $new_time = $timetrackerService->convertTimestamp($current_total_time + $extra_time);
    $ajax_response->addCommand(new HtmlCommand('h4.modal-title', 'Total tracked time: ' . $new_time));

    // Update the total as well.
    $ajax_response->addCommand(new HtmlCommand('.timetracker-total-value', $new_time));

    // Todo - Fix me (no wrapper div).
    /* // Create the html for the table row.
    $row_html = [
      '#type' => 'template',
      '#theme' => 'timetracker_table_row',
      '#vars' => [
        'date' => $timetrackerService->convertTimestamp($start),
        'user' => \Drupal::currentUser()->getDisplayName(),
        'message' => $message,
        'time' => $timetrackerService->convertTimestamp($extra_time)
      ],
      '#pre_render' => [],
      '#theme_wrappers' => [],
    ];

    $row = \Drupal::service('renderer')->render($row_html);*/

    // Add the row to the table.
    $ajax_response->addCommand(new BeforeCommand('.timetracker-total',
      '<tr class="timetracker-row with-cookies">
      <td class="timetracker-row-date">' . $timetrackerService->convertTimestamp($start, 'date', '/') . ' </td>
      <td class="timetracker-row-user">' . \Drupal::currentUser()
        ->getDisplayName() . ' </td>
      <td class="timetracker-row-message">' . $message . ' </td>
      <td class="timetracker-row-time">' . $timetrackerService->convertTimestamp($extra_time) . ' </td>
      </tr>'));

    // Show the status messages.
    /*$status_messages = array('#type' => 'status_messages');
    $status_messages = \Drupal::service('renderer')->render($status_messages);
    $ajax_response->addCommand(new HtmlCommand('.body', $status_messages));*/

    return $ajax_response;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {

    if (!$form_state->getValue('timetracker_message') || empty($form_state->getValue('timetracker_message'))) {
      $form_state->setErrorByName('timetracker_message', $this->t('Please provide a message.'));
    }

    parent::validateForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    // Display result.
    foreach ($form_state->getValues() as $key => $value) {
      drupal_set_message($key . ': ' . $value);
    }

  }

}
