<?php

namespace Drupal\openlucius_timetracker;

use DateTime;
use DateTimeZone;
use Drupal\Core\Database\Driver\mysql\Connection;
use Drupal\user\Entity\User;
use PDOException;

/**
 * Class TimeService.
 *
 * @package Drupal\openlucius_timetracker
 */
class TimetrackerService implements TimetrackerInterface {

  /**
   * Drupal\Core\Database\Driver\mysql\Connection definition.
   *
   * @var \Drupal\Core\Database\Driver\mysql\Connection
   */
  protected $database;

  /**
   * Constructs a new TestService object.
   *
   * @param \Drupal\Core\Database\Driver\mysql\Connection $database
   */
  public function __construct(Connection $database) {
    $this->database = $database;
  }

  /**
   * Creates a time entry.
   *
   * @param int $nid
   *   The current node id.
   * @param int $uid
   *   The current user id.
   * @param int $start
   *   The start timestamp.
   * @param int $stop
   *   The stop timestamp.
   * @param string $message
   *   The message.
   *
   * @return array
   *   Returns the message.
   */
  public function createTimeEntry($nid, $uid, $start, $stop, $message) {

    try {

      // Temporarily truncate the watchdog database.
      db_truncate('watchdog')->execute();

      // Insert the time entry.
      $this->database->insert('ol_time_entries')->fields([
        'tid' => NULL,
        'uid' => $uid,
        'nid' => $nid,
        'start' => $start,
        'stop' => $stop,
        'message' => $message,
      ])->execute();
    }
    catch (PDOException $e) {
      \Drupal::logger('create_time_entry')
        ->notice('PDOException @e', ['@e' => $e->getMessage()]);
    }

    // Always return a response.
    return new JsonResponse([
      'entry' => 'entry',
    ]);
  }

  /**
   * Create / Update the day totals.
   *
   * @param int $uid
   *   The user id.
   * @param int $extra_time
   *   The unix timestamp to add.
   */
  public function createUserDayTotal($uid, $extra_time) {

    // Get the first second of today.
    $today = $this->generateTimestamp();

    // Get the entries for the user.
    $entry = $this->readUserDayTotal($uid, $today['start_today'], $today['end_today']);

    // Update the record if it exists.
    if (!empty($entry->tid)) {
      $this->UpdateUserDayTotal($uid, $extra_time);
    }
    else {
      try {
        // Insert the time entry.
        $this->database->insert('ol_time_entries_day_totals')->fields([
          'uid' => $uid,
          'date' => $today['start_today'],
          'time' => $extra_time,
        ])->execute();
      }
      catch (PDOException $e) {
        \Drupal::logger('create_user_total')
          ->notice('PDOException @e', ['@e' => $e->getMessage()]);
      }
    }
  }

  /**
   * Reads a time entry.
   *
   * @param int $nid
   *   The current node id.
   * @param int $uid
   *   The user id.
   *
   * @return array
   *   Returns the time entries as array.
   */
  public function readTimeEntry($nid, $uid = NULL) {

    // Return an empty array if there is no node id.
    if (empty($nid)) {
      return [];
    }

    // Query the time entry table.
    $query = $this->readTimeEntryQuery($nid);

    if (!empty($uid)) {

      // Query the time entry table.
      $query = $this->readTimeEntryQuery($nid, $uid);
    }

    // Get the data.
    $data = $query->execute()->fetchAll();

    // Return the data.
    return $data;
  }

  /**
   * Get a day total for a specific user.
   *
   * @param int $uid
   *   The user id.
   * @param int $from
   *   The start timestamp
   * @param int $to
   *   The end timestamp
   *
   * @return array
   */
  public function readUserDayTotal($uid, $from = NULL, $to = NULL) {

    // Return an empty array if there is no node id.
    if (empty($uid)) {
      return [];
    }

    // Query the time entry table.
    $query = $this->readUserDayTotalQuery($uid, $from, $to);

    // Get the data.
    $data = $query->execute()->fetchObject();

    // Return the data.
    return $data;
  }

  /**
   * Updates a time entry.
   *
   * @param int $nid
   *   The current node id.
   * @param int $uid
   *   The current user id.
   */
  public function updateTimeEntry($nid, $uid) {

    // Get the time entry.
    $entry = $this->readTimeEntry($nid);

    if (!empty($entry['tid'])) {
      $this->database->update('ol_time_entries')
        ->fields([
          'stop' => time(),
        ])
        ->condition('tid', $entry['tid'])
        ->execute();
    }

    // TODO: Implement updateTimeEntry() method.
    \Drupal::logger('timetracker')
      ->notice('Update trigger for %id', ['%id' => $entry['tid']]);
  }

  /**
   * Updates a time entry.
   *
   * @param int $uid
   *   The current user id.
   * @param int $time
   *   The extra time as unix timestamp.
   */
  public function UpdateUserDayTotal($uid, $time) {

    // Get the first second of today.
    $today = $this->generateTimestamp();

    // Get the entries for the user.
    $entry = $this->readUserDayTotal($uid, $today['start_today'], $today['end_today']);

    if (!empty($entry->tid)) {

      try {
        $this->database->update('ol_time_entries_day_totals')
          ->fields([
            'time' => $entry->time + $time,
          ])
          ->condition('uid', $uid)
          ->execute();
      }
      catch (PDOException $e) {
        \Drupal::logger('create_user_total')
          ->notice('PDOException @e', ['@e' => $e->getMessage()]);
      }
    }
  }

  /**
   * Deletes a time entry.
   *
   * @param int $nid
   *   The current node id.
   * @param int $uid
   *   The current user id.
   */
  public function deleteTimeEntry($nid, $uid) {
    // TODO: Implement deleteTimeEntry() method.
    \Drupal::logger('tracker')
      ->notice('Calling function: %function', ['%function' => 'deleteTimeEntry']);
  }

  /**
   * End CRUD.
   */

  /**
   * @param $nid
   * @param $uid
   *
   * @param string $format
   *
   * @return string
   */
  public function getMyTime($nid, $uid = NULL, $format = 'plain') {

    $my_time = 0;

    $my_time_entries = $this->readTimeEntry($nid, $uid);

    foreach ($my_time_entries as $entry) {
      $my_time += ($entry->stop - $entry->start);
    }

    if ($format == 'plain') {
      return $my_time;
    }

    if ($format == 'formatted') {
      return $this->convertTimestamp($my_time, 'cumulative', ':');
    }

    return '';
  }

  /**
   * Get the query for the time entries.
   *
   * @param int $nid
   *   The node id.
   * @param null $uid
   *   The user id.
   * @param null $from
   *   The start time as unix timestamp.
   * @param null $to
   *   The stop time as unix timestamp
   *
   * @return \Drupal\Core\Database\Query\SelectInterface
   */
  public function readTimeEntryQuery($nid, $uid = NULL, $from = NULL, $to = NULL) {

    // Query the time entry table.
    $query = $this->database->select('ol_time_entries', 'o');

    // Select all fields.
    $query->fields('o');
    $query->condition('o.nid', $nid, '=');

    // Check if we need to filter on a user id.
    if (!empty($uid)) {
      $query->condition('o.uid', $uid, '=');
    }

    if (!empty($from)) {
      $query->condition('o.start', $from, '>');
    }

    if (!empty($to)) {
      $query->condition('o.stop', $to, '<');
    }

    // Return the query.
    return $query;
  }

  /**
   * Get the query for the user day totals.
   *
   * @param int $uid
   *   The user id.
   * @param null $from
   *   The start time as unix timestamp.
   * @param null $to
   *   The stop time as unix timestamp
   *
   * @return \Drupal\Core\Database\Query\SelectInterface
   */
  public function readUserDayTotalQuery($uid, $from = NULL, $to = NULL) {

    // Query the time entry table.
    $query = $this->database->select('ol_time_entries_day_totals', 't');

    // Select all fields.
    $query->fields('t');
    $query->condition('t.uid', $uid, '=');

    if (!empty($from) && !empty($to)) {
      $query->condition('t.date', [$from, $to], 'BETWEEN');
    }

    // Return the query.
    return $query;
  }

  /**
   * End of CRUD.
   */

  /**
   * Get the total tracked time for a node.
   *
   * @param int $nid
   *   The node id.
   * @param null $from
   *   The start time as unix timestamp.
   * @param null $to
   *   The stop time as unix timestamp
   *
   * @return int
   *   Returns the total time as integer.
   */
  public function getTotalTime($nid, $from = NULL, $to = NULL) {

    // Query the time entry table.
    $query = $this->readTimeEntryQuery($nid);

    // Check for a range.
    if (!empty($from) && !empty($to)) {
      $query = $this->readTimeEntryQuery($nid, $from, $to);
    }

    // Get the data.
    $data = $query->execute()->fetchAll();

    // Initialize the total time that will be added in the loop.
    $total_time = 0;

    foreach ($data as $entry) {

      // Add the time for each record to the total.
      $total_time += $entry->stop - $entry->start;
    }

    return $total_time;
  }

  /**
   * Get the formatted total tracked time for a node.
   *
   * @param int $nid
   *   The node id.
   *
   * @return string
   *   Returns the total time as formatted time string.
   */
  public function getFormattedTotalTime($nid) {
    $total_time = $this->getTotalTime($nid);
    return $this->convertTimestamp($total_time);
  }

  /**
   * @param array $entry
   *   The time entry array.
   *
   * @return string
   *   Returns the formatted timestamp.
   */
  public function formatTimeEntryTimestamp($entry) {

    // Return empty string if no entry.
    if (empty($entry)) {
      return '';
    }

    // Set a prefix to make the width fit.
    $prefix = '';

    // Get the absolute value.
    $time_entry = abs($entry->stop - $entry->start);

    // Convert the timestamp.
    $time = $this->convertTimestamp($time_entry, 'time', ':');

    // Add a minus if below zero.
    if (($entry->stop - $entry->start) < 0) {
      $prefix = '- ';
    }

    return $prefix . $time;
  }

  /**
   * Get the current node id.
   */
  public function getCurrentNodeId() {

    $id = FALSE;

    // Get the internal path.
    $current_path = \Drupal::service('path.current')->getPath();

    // Retrieve an array which contains the path pieces.
    $pieces = explode('/', $current_path);

    foreach ($pieces as $key => $p) {
      if ($p == 'channel') {
        $id = $pieces[$key + 1];
      }
    }

    // Return the id.
    return $id;
  }

  /**
   * Get a username form a uid.
   *
   * @param int $uid
   *   The user id.
   *
   * @return string
   *   Returns the username as string.
   */
  public function getUserName($uid) {
    $account = User::load($uid);
    return $account->getUsername();
  }

  /**
   * Convert a unix timestamp.
   *
   * @param int $timestamp
   *   The timestamp to convert.
   * @param string $type
   *   The type to convert the timestamp to.
   * @param string $separator
   *   The separator string.
   *
   * @return string
   *   Returns a date/time as string.
   */
  public function convertTimestamp($timestamp, $type = 'time', $separator = ':') {

    // Create a new DateTime.
    $dt = new DateTime('@' . $timestamp, new DateTimeZone('Europe/Amsterdam'));

    if ($type == 'time') {
      return $dt->format('H') . $separator . $dt->format('i') . $separator . $dt->format('s');
    }

    if ($type == 'date') {
      return $dt->format('d') . $separator . $dt->format('m') . $separator . $dt->format('Y');
    }

    if ($type == 'chart') {
      return $dt->format('G') . $separator . $dt->format('i');
    }

    if ($type == 'cumulative') {

      return $dt->format('H') . $separator . $dt->format('i') . $separator . $dt->format('s');
      //return str_pad(floor($timestamp / 3600), 2, '0', STR_PAD_LEFT) . gmdate(":i", $timestamp % 3600);
    }

    return '';
  }

  /**
   * Generate an array of timestamps.
   *
   * @return array
   *   Returns an array of timestamps.
   */
  public function generateTimestamp() {

    $start_today = new DateTime ("midnight today");
    $end_today = new DateTime ("midnight tomorrow");
    $end_today->modify('1 second ago');

    $oToday = new DateTime();
    $iTime = mktime(0, 0, 0, $oToday->format('m'), 1, $oToday->format('Y'));
    $oFirst = new DateTime(date('r', $iTime));

    $oLast = clone $oFirst;
    $oLast->modify('+1 month');
    $oLast->modify('-1 day');
    $oLast->setTime(23, 59, 59);

    return [
      'start_today' => $start_today->getTimestamp(),
      'end_today' => $end_today->getTimestamp(),
      'start_month' => $oFirst->getTimestamp(),
      'end_month' => $oLast->getTimestamp(),
    ];
  }

  /**
   * Get the timestamp for the start of today.
   *
   * @return array
   *   Returns the timestamp array.
   */
  public function getTodayTimestamp() {

    $dtNow = new DateTime();
    // Set a non-default timezone if needed
    $dtNow->setTimezone(new DateTimeZone('Europe/Amsterdam'));
    $dtNow->setTimestamp(time());

    $beginOfDay = clone $dtNow;

    // Go to midnight.  ->modify('midnight') does not do this for some reason.
    $beginOfDay->modify('today');

    $endOfDay = clone $beginOfDay;
    $endOfDay->modify('tomorrow');
    // Adjust from the next day to the end of the day, per original question.
    $endOfDay->modify('1 second ago');

    return [
      'time ' => $dtNow->getTimestamp(),
      'start' => $beginOfDay->getTimestamp(),
      'end' => $endOfDay->getTimestamp(),
    ];
  }

  /**
   * Get the timestamp for the start of this month.
   *
   * @return int
   *   Returns the timestamp as int.
   */
  public function getStartOfMonthTimestamp() {

    // Create the from and to timestamps for this month.
    $dt = new DateTime();
    $format = mktime(0, 0, 0, $dt->format('m'), 1, $dt->format('Y'));
    $time = new DateTime(date('r', $format));

    return $time->getTimestamp();
  }

  /**
   * Get the timestamp for the end of this month.
   *
   * @return int
   *   Returns the timestamp as int.
   */
  public function getEndOfMonthTimestamp() {

    // Create the from and to timestamps for this month.
    $dt = new DateTime();
    $format = mktime(0, 0, 0, $dt->format('m'), 1, $dt->format('Y'));
    $time = new DateTime(date('r', $format));

    $to_time = clone $time;
    $to_time->modify('+1 month');
    $to_time->modify('-1 day');
    $to_time->setTime(23, 59, 59);

    return $to_time->getTimestamp();
  }

  /**
   * Get the day totals withing a specific range.
   *
   * @param int $uid
   *   The user id.
   * @param int $from
   *   The start time as unix timestamp.
   * @param $to
   *   The end time as unix timestamp.
   *
   * @return mixed
   */
  public function getDayTotals($uid, $from, $to) {

    $data = [];

    $query = $this->readUserDayTotalQuery($uid, $from, $to);
    $entries = $query->execute()->fetchAll();

    foreach ($entries as $entry) {
      $day = $this->convertTimestamp($entry->date, 'date', '/');
      $data[$day] = $this->convertTimestamp($entry->time, 'chart', '.');
    }

    return $data;
  }

  /**
   * Get the user totals withing a specific range.
   *
   * @param int $uid
   *   The user id.
   * @param int $from
   *   The start time as unix timestamp.
   * @param $to
   *   The end time as unix timestamp.
   *
   * @return mixed
   */
  public function getUserTotals($uid, $from, $to) {

    /**
     * Get all entries from ol_time_entries_day_totals.
     * Start and end should be from filter. Default to this month.
     */

    $query = $this->readUserDayTotalQuery($uid, $from, $to);
    $entries = $query->execute()->fetchAll();

    $sum = 0;

    foreach ($entries as $entry) {
      $sum += $entry->time;
    }

    // Get the hours minutes seconds.
    $data = $this->convertTimestamp($sum, 'cumulative');

    return $data;
  }

  /**
   * Get all users.
   *
   * @return mixed
   *   Returns the users.
   */
  public function getUsers() {

    $items = [];

    // Query users in channel.
    $query = \Drupal::database()->select('ol_user_channel', 'nfd');
    $query->addField('ufd', 'name', 'label');
    $query->addField('nfd', 'uid', 'id');
    $query->join('users_field_data', 'ufd', 'ufd.uid = nfd.uid');

    $users = $query->execute()->fetchAll();

    foreach ($users as $u) {
      $items[$u->id] = $u->label;
    }

    return $items;
  }

}
