<?php

namespace Drupal\openlucius_timetracker;

/**
 * Class TimeService.
 *
 * @package Drupal\openlucius_timetracker
 */
class TimetrackerTableService extends TimetrackerService implements TimetrackerTableInterface {

  /**
   * Create the table with time entries for a node.
   *
   * @param int $nid
   *   The node id.
   *
   * @return string
   *   Returns the time entries as html table or empty string.
   */
  public function getTimeEntriesTable($nid) {

    // Get all time entries for the node.
    $entries = $this->readTimeEntry($nid);

    // Create the table.
    $table_array = $this->createTimeEntriesTable($entries);

    // Return the rendered output.
    return \Drupal::service('renderer')->render($table_array);
  }

  /**
   * Get the table rows.
   *
   * @param array $entries
   *   The time entries.
   *
   * @return array
   *   Returns the table as array.
   */
  public function createTimeEntriesTable($entries) {

    // Create the table header.
    $header = $this->createTimeEntriesTableHeader();
    $rows = [];

    // Initialize the total time that will be added in the loop.
    $total_time = 0;

    foreach ($entries as $entry) {

      // Create the table row.
      $row = $this->createTimeEntriesTableRow($entry);

      // Add the row.
      $rows[] = $row;

      // Add the time for each record to the total.
      $total_time += $entry->stop - $entry->start;
    }

    // Add the total row.
    $this->createTimeEntriesTableRowTotal($rows, $total_time);

    // Create the table.
    $table = [
      '#type' => 'table',
      '#header' => $header,
      '#rows' => $rows,
      '#attributes' => [
        'class' => [
          'table',
          'table-bordered',
        ],
      ],
    ];

    // Return the table.
    return $table;
  }

  /**
   * Create the table header.
   *
   * @return array
   *   Returns the table header as array.
   */
  public function createTimeEntriesTableHeader() {

    return [
      [
        'data' => t('Date'),
        'align' => 'left',
      ],
      [
        'data' => t('User'),
        'align' => 'center',
      ],
      [
        'data' => t('Message'),
        'align' => 'center',
      ],
      [
        'data' => t('Time'),
        'align' => 'center',
      ],
    ];
  }

  /**
   * Create the row.
   *
   * @param array $entry
   *   The time entry array.
   *
   * @return array
   *   Returns the table row as array.
   */
  public function createTimeEntriesTableRow($entry) {

    // Create the row.
    return [
      'data' => [
        'date' => [
          'data' => $this->convertTimestamp($entry->start, 'date', '/'),
          'class' => 'timetracker-row-date',
        ],
        'user' => [
          'data' => $this->getUserName($entry->uid),
          'class' => 'timetracker-row-user',
        ],
        'message' => [
          'data' => $entry->message,
          'class' => 'timetracker-row-message',
        ],
        'time' => [
          'data' => $this->formatTimeEntryTimestamp($entry),
          'class' => 'timetracker-row-time',
        ],
      ],
      'class' => 'timetracker-row',
    ];
  }

  /**
   * Create the total row.
   */
  public function createTimeEntriesTableRowTotal(&$rows, $total_time) {

    // Add the total.
    $rows[] = [
      'data' => [
        'date' => [
          'data' => t('Total'),
          'class' => [
            'timetracker-total-label',
          ],
          'colspan' => 3,
        ],
        'time' => [
          'data' => $this->convertTimestamp($total_time, 'time', ':'),
          'class' => [
            'timetracker-total-value',
          ],
        ],
      ],
      'class' => 'timetracker-total',
    ];

  }
}
