<?php

/**
 * @file
 * Enables modules and site configuration for a openlucius site installation.
 */
use Drupal\Core\Form\FormStateInterface;
use Drupal\user\Entity\User;

/**
 * Implements hook_install_tasks().
 */
function openlucius_install_tasks(&$install_state) {
  $tasks = array(
    'openlucius_install_profile_modules' => array(
      'display_name' => t('Install OpenLucius modules'),
      'type' => 'batch',
    ),
    'openlucius_final_site_setup' => array(
      'display_name' => t('Apply configuration'),
      'type' => 'batch',
      'display' => TRUE,
    ),
  );
  return $tasks;
}

/**
 * Implements hook_install_tasks_alter().
 */
function openlucius_install_tasks_alter(&$tasks, $install_state) {
  $tasks['install_finished']['function'] = 'openlucius_install_finished';
}

/**
 * Installs required modules via a batch process.
 *
 * @param $install_state
 *   An array of information about the current installation state.
 *
 * @return
 *   The batch definition.
 */
function openlucius_install_profile_modules(&$install_state) {

  $files = system_rebuild_module_data();

  $modules = array(
    'openlucius_channel' => 'openlucius_channel',
    'openlucius_message' => 'openlucius_message',
    'openlucius_core' => 'openlucius_core',
    'openlucius_config' => 'openlucius_config',
    'openlucius_usermanager' => 'openlucius_usermanager',
    'openluciustheme_profile_settings' => 'openluciustheme_profile_settings',
  );

  $openlucius_modules = $modules;

  // Install required modules first.
  $required = array();
  $non_required = array();

  // Add modules that other modules depend on.
  foreach ($modules as $module) {
    if ($files[$module]->requires) {
      $module_requires = array_keys($files[$module]->requires);
      // Remove the openlucius modules from required modules.
      $module_requires = array_diff_key($module_requires, $openlucius_modules);
      $modules = array_merge($modules, $module_requires);
    }
  }
  $modules = array_unique($modules);
  // Remove the openlucius modules from to install modules.
  $modules = array_diff_key($modules, $openlucius_modules);
  foreach ($modules as $module) {
    if (!empty($files[$module]->info['required'])) {
      $required[$module] = $files[$module]->sort;
    }
    else {
      $non_required[$module] = $files[$module]->sort;
    }
  }
  arsort($required);

  $operations = array();
  foreach ($required + $non_required + $openlucius_modules as $module => $weight) {
    $operations[] = array(
      '_openlucius_install_module_batch',
      array(array($module), $module)
    );
  }

  $batch = array(
    'operations' => $operations,
    'title' => t('Install OpenLucius modules'),
    'error_message' => t('The installation has encountered an error.'),
  );


  return $batch;
}

/**
 * Implements callback_batch_operation().
 *
 * Performs batch installation of modules.
 */
function _openlucius_install_module_batch($module, $module_name, &$context) {
  set_time_limit(0);
  \Drupal::service('module_installer')->install($module, $dependencies = TRUE);
  $context['results'][] = $module;
  $context['message'] = t('Install %module_name module.', array('%module_name' => $module_name));
}


/**
 * @param $install_state
 */
function openlucius_final_site_setup(&$install_state) {

  // Clear all status messages generated by modules installed in previous step.
  drupal_get_messages('status', TRUE);

  node_access_needs_rebuild(FALSE); // There is no content at this point.

  $batch = array();

  return $batch;
}

/**
 * Performs final installation steps and displays a 'finished' page.
 *
 * @param $install_state
 *   An array of information about the current installation state.
 *
 * @return
 *   A message informing the user that the installation is complete.
 *
 * @see install_finished().
 */
function openlucius_install_finished(&$install_state) {
  // Clear all status messages generated by modules installed in previous step.
  drupal_get_messages('status', TRUE);
  if ($install_state['interactive']) {
    // Load current user and perform final login tasks.
    // This has to be done after drupal_flush_all_caches()
    // to avoid session regeneration.
    $account = User::load(1);
    user_login_finalize($account);

  }

}